/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.process;

import org.opengion.fukurou.util.Argument;
import org.opengion.fukurou.util.HybsEntry ;
import org.opengion.fukurou.util.FileUtil;
import org.opengion.fukurou.util.Closer ;
import org.opengion.fukurou.util.LogWriter;

import java.util.Map ;
import java.util.LinkedHashMap ;

import java.io.File;
import java.io.PrintWriter;

/**
 * Process_TableWriter は、上流から受け取ったデータをファイルに書き込む
 * CainProcess インターフェースの実装クラスです。
 *
 * 上流(プロセスチェインのデータは上流から下流へと渡されます。)から
 * 受け取ったLineModel を元に、DBTableModel 形式ファイルを出力します。
 *
 * 引数文字列中にスペースを含む場合は、ダブルコーテーション("") で括って下さい。
 * 引数文字列の 『=』の前後には、スペースは挟めません。必ず、-key=value の様に
 * 繋げてください。
 *
 * @og.formSample
 *  Process_TableWriter -outfile=OUTFILE -sep=, -encode=UTF-8 -append=true
 *
 *    -outfile=出力ﾌｧｲﾙ名       ：出力ﾌｧｲﾙ名
 *   [-sep=ｾﾊﾟﾚｰﾀ文字         ] ：区切り文字(初期値:タブ)
 *   [-encode=文字ｴﾝｺｰﾄﾞ      ] ：出力ﾌｧｲﾙのｴﾝｺｰﾄﾞﾀｲﾌﾟ
 *   [-append=[false/true]    ] ：出力ﾌｧｲﾙを、追記する(true)か新規作成する(false)か。
 *   [-useHeader=[true/false] ] ：ﾍｯﾀﾞｰ情報(#NAME行)を出力する(true)か出力しない(false)か。
 *   [-useNumber=[true/false] ] ：行番号を出力する(true)か出力しない(false)か。
 *   [-useWquot=[false/true]  ] ：出力ﾃﾞｰﾀをﾀﾞﾌﾞﾙｸｵｰﾃｰｼｮﾝで括る(true)かそのまま(false)か。
 *   [-omitCTRL=[false/true]  ] ：ｺﾝﾄﾛｰﾙ文字を削除する(true)かそのまま(false)か。
 *   [-const_XXXX=固定値      ] ：-const_FGJ=1
 *                                    LineModel のキー(const_ に続く文字列)の値に、固定値を設定します。
 *                                   キーが異なれば、複数のカラム名を指定できます。
 *   [-display=[false/true]   ] ：結果を標準出力に表示する(true)かしない(false)か(初期値:false[表示しない])
 *   [-debug=[false/true]     ] ：デバッグ情報を標準出力に表示する(true)かしない(false)か(初期値:false[表示しない])
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class Process_TableWriter extends AbstractProcess implements ChainProcess {
	private static final String CNST_KEY = "const_" ;

	private String	outfile		= null;
	private PrintWriter writer	= null;
	private String	separator	= TAB;	// 項目区切り文字

	private String[]	cnstClm		= null;		// 固定値を設定するカラム名
	private int[]		cnstClmNos	= null;		// 固定値を設定するのカラム番号
	private String[]	constVal	= null;		// カラム番号に対応した固定値
	private File		file		= null;		// 出力ファイル
	private String		encode		= System.getProperty("file.encoding");	// 出力ファイルエンコード
	private boolean		append		= false;	// ファイル追加(true:追加/false:通常)
	private boolean		useHeader	= true;		// ヘッダー情報(#NAME行)を出力する(true)か出力しない(false)か。
	private boolean		useNumber	= true;		// 行番号を出力する(true)か出力しない(false)か。
	private boolean		useWquot	= false;	// 出力データをダブルクオーテーションで括る(true)かそのまま(false)か。
	private boolean		omitCTRL	= false;	// コントロール文字を削除する(true)かそのまま(false)か。
	private boolean		display		= false;	// 表示しない
	private boolean		debug		= false;	// 5.7.3.0 (2014/02/07) デバッグ情報

	private boolean firstRow	= true;	// 最初の一行目
	private int		count		= 0;

	private static final Map<String,String> mustProparty   ;		// ［プロパティ］必須チェック用 Map
	private static final Map<String,String> usableProparty ;		// ［プロパティ］整合性チェック Map

	static {
		mustProparty = new LinkedHashMap<String,String>();
		mustProparty.put( "outfile",	"出力ﾌｧｲﾙ名 (必須)" );

		usableProparty = new LinkedHashMap<String,String>();
		usableProparty.put( "sep",		"区切り文字(初期値:ﾀﾌﾞ)" );
		usableProparty.put( "encode",	"出力ﾌｧｲﾙのｴﾝｺｰﾄﾞﾀｲﾌﾟ" );
		usableProparty.put( "append",	"出力ﾌｧｲﾙを、追記する(true)か新規作成する(false)か。" );
		usableProparty.put( "useHeader",	"ﾍｯﾀﾞｰ情報(#NAME行)を出力する(true)か出力しない(false)か。" );
		usableProparty.put( "useNumber",	"行番号を出力する(true)か出力しない(false)か。" );
		usableProparty.put( "useWquot",		"出力ﾃﾞｰﾀをﾀﾞﾌﾞﾙｸｵｰﾃｰｼｮﾝで括る(true)かそのまま(false)か。" );
		usableProparty.put( "omitCTRL",		"ｺﾝﾄﾛｰﾙ文字を削除する(true)かそのまま(false)か。" );
		usableProparty.put( "const_",	"LineModel のキー(const_ に続く文字列)の値に、固定値を" +
									CR + "設定します。キーが異なれば、複数のカラム名を指定できます。" +
									CR + "例: -const_FGJ=1" );
		usableProparty.put( "display",	"結果を標準出力に表示する(true)かしない(false)か" +
									CR + " (初期値:false:表示しない)" );
		usableProparty.put( "debug",	"デバッグ情報を標準出力に表示する(true)かしない(false)か" +
									CR + "(初期値:false:表示しない)" );		// 5.7.3.0 (2014/02/07) デバッグ情報
	}

	/**
	 * デフォルトコンストラクター。
	 * このクラスは、動的作成されます。デフォルトコンストラクターで、
	 * super クラスに対して、必要な初期化を行っておきます。
	 *
	 */
	public Process_TableWriter() {
		super( "org.opengion.fukurou.process.Process_TableWriter",mustProparty,usableProparty );
	}

	/**
	 * プロセスの初期化を行います。初めに一度だけ、呼び出されます。
	 * 初期処理(ファイルオープン、ＤＢオープン等)に使用します。
	 *
	 * @param   paramProcess データベースの接続先情報などを持っているオブジェクト
	 */
	public void init( final ParamProcess paramProcess ) {
		Argument arg = getArgument();

		outfile			= arg.getProparty("outfile");
		encode	 		= arg.getProparty("encode",encode);
		separator		= arg.getProparty("sep",separator );
		append			= arg.getProparty("append",append);
		useHeader		= arg.getProparty("useHeader",useHeader);
		useNumber		= arg.getProparty("useNumber",useNumber);
		useWquot		= arg.getProparty("useWquot",useWquot);
		omitCTRL		= arg.getProparty("omitCTRL",omitCTRL);
		HybsEntry[] cnstKey = arg.getEntrys( CNST_KEY );		// 配列
		display			= arg.getProparty("display",display);
		debug			= arg.getProparty("debug",debug);				// 5.7.3.0 (2014/02/07) デバッグ情報

		int size   = cnstKey.length;
		cnstClm    = new String[size];
		constVal   = new String[size];
		for( int i=0; i<size; i++ ) {
			cnstClm[i]  = cnstKey[i].getKey();
			constVal[i] = cnstKey[i].getValue();
		}

		if( outfile == null ) {
			String errMsg = "ファイル名が指定されていません。" ;
			throw new RuntimeException( errMsg );
		}

		file = new File( outfile );
		File dir = file.getParentFile() ;

		// ディレクトリが存在しない場合の処理
		if( ! dir.exists() && ! dir.mkdirs() ) {
			String errMsg = "ディレクトリが作成できませんでした。[" + dir + "]" ;
			throw new RuntimeException( errMsg );
		}
	}

	/**
	 * プロセスの終了を行います。最後に一度だけ、呼び出されます。
	 * 終了処理(ファイルクローズ、ＤＢクローズ等)に使用します。
	 *
	 * @param   isOK トータルで、OKだったかどうか[true:成功/false:失敗]
	 */
	public void end( final boolean isOK ) {
		if( writer != null ) {
			writer.flush();
			Closer.ioClose( writer );
			writer = null;
		}
	}

	/**
	 * 引数の LineModel を処理するメソッドです。
	 * 変換処理後の LineModel を返します。
	 * 後続処理を行わない場合(データのフィルタリングを行う場合)は、
	 * null データを返します。つまり、null データは、後続処理を行わない
	 * フラグの代わりにも使用しています。
	 * なお、変換処理後の LineModel と、オリジナルの LineModel が、
	 * 同一か、コピー(クローン)かは、各処理メソッド内で決めています。
	 * ドキュメントに明記されていない場合は、副作用が問題になる場合は、
	 * 各処理ごとに自分でコピー(クローン)して下さい。
	 *
	 * @param   data	オリジナルのLineModel
	 *
	 * @return	処理変換後のLineModel
	 */
	public LineModel action( final LineModel data ) {
		count++ ;
	//	if( display ) { println( data.dataLine() ); }
		if( firstRow ) {
			writer = FileUtil.getPrintWriter( file,encode,append );
			if( useHeader && useNumber ) { writeName( data ); }

			int size   = cnstClm.length;
			cnstClmNos = new int[size];
			for( int i=0; i<size; i++ ) {
				cnstClmNos[i] = data.getColumnNo( cnstClm[i] );
			}

			firstRow = false;
			if( display ) { println( data.nameLine() ); }		// 5.7.3.0 (2014/02/07) デバッグ情報
		}

		// 固定値置き換え処理
		for( int j=0; j<cnstClmNos.length; j++ ) {
			data.setValue( cnstClmNos[j],constVal[j] );
		}

		writeData( data );

		if( display ) { println( data.dataLine() ); }	// 5.1.2.0 (2010/01/01) display の条件変更
		return data;
	}

	/**
	 * PrintWriter に LineModelの項目名情報を書き込みます。
	 * 第一カラム目は、項目名情報を示す "#Name" を書き込みます。
	 * この行は、出力形式に無関係に、TAB で区切られます。
	 *
	 * @param	data ラインモデル
	 */
	private void writeName( final LineModel data ) {
		int size = data.size();
		writer.print( "#Name" );
		for( int clm=0; clm<size; clm++ ) {
			writer.print( TAB );
			writer.print( data.getName(clm) );
		}
		writer.println();
	}

	/**
	 * PrintWriter に LineModelのテーブル情報を書き込みます。
	 *
	 * @og.rev 5.2.2.0 (2010/11/01) 改行を含む場合は、ダブルクオートを強制的に前後に追加する。
	 * @og.rev 5.2.2.0 (2010/11/01) ダブルクオートを含む場合は、その直前にダブルクオートを強制的に追加する。
	 *
	 * @param	data ラインモデル
	 */
	private void writeData( final LineModel data ) {
		int size = data.size();

		if( useNumber ) { writer.print( data.getRowNo() ); }		// 行番号
		for( int clm=0; clm<size; clm++ ) {
			if( useNumber || clm!=0 ) { writer.print( separator ); }
			Object val = data.getValue(clm);
			if( val == null ) { val = ""; }

			String sval = String.valueOf( val );
			// 5.2.2.0 (2010/11/01) ダブルクオートを含む場合は、その直前にダブルクオートを強制的に追加する。
			if( sval.indexOf( '"' ) >= 0 ) { sval = sval.replaceAll( "\"" ,"\"\"" ) ; }
			if( omitCTRL ) { sval = sval.replaceAll( "\\s" ," " ) ; }
			// 5.2.2.0 (2010/11/01) 改行を含む場合は、ダブルクオートを強制的に前後に追加する。
			if( useWquot || ( !omitCTRL && sval.indexOf( CR ) >= 0 ) ) {
				sval = "\"" + sval + "\"" ;
			}
			writer.print( sval );
		}
		writer.println();
	}

	/**
	 * プロセスの処理結果のレポート表現を返します。
	 * 処理プログラム名、入力件数、出力件数などの情報です。
	 * この文字列をそのまま、標準出力に出すことで、結果レポートと出来るような
	 * 形式で出してください。
	 *
	 * @return   処理結果のレポート
	 */
	public String report() {
		String report = "[" + getClass().getName() + "]" + CR
				+ TAB + "Output File  : " + outfile + CR
				+ TAB + "Output Count : " + count ;

		return report ;
	}

	/**
	 * このクラスの使用方法を返します。
	 *
	 * @return	このクラスの使用方法
	 */
	public String usage() {
		StringBuilder buf = new StringBuilder();

		buf.append( "Process_TableWriter は、上流から受け取ったデータをファイルに書き込む" 			).append( CR );
		buf.append( "CainProcess インターフェースの実装クラスです。"								).append( CR );
		buf.append( CR );
		buf.append( "上流(プロセスチェインのデータは上流から下流へと渡されます。)から"			).append( CR );
		buf.append( "受け取ったLineModel を元に、DBTableModel 形式ファイルを出力します。"			).append( CR );
		buf.append( CR );
		buf.append( "引数文字列中に空白を含む場合は、ダブルコーテーション(\"\") で括って下さい。"	).append( CR );
		buf.append( "引数文字列の 『=』の前後には、空白は挟めません。必ず、-key=value の様に"		).append( CR );
		buf.append( "繋げてください。"																).append( CR );
		buf.append( CR ).append( CR );

		buf.append( getArgument().usage() ).append( CR );

		return buf.toString();
	}

	/**
	 * このクラスは、main メソッドから実行できません。
	 *
	 * @param	args	コマンド引数配列
	 */
	public static void main( final String[] args ) {
		LogWriter.log( new Process_TableWriter().usage() );
	}
}
